<?php
/*
Plugin Name: CSV Im- & Export
Description: Importiert und exportiert CSV-Daten eiens ausgewählten Post Types.
Version: 0.1
Author: Fabian Freitag
*/

if (!defined('ABSPATH')) {
    exit; // Sicherheitshalber verhindern, dass das Plugin direkt aufgerufen wird
}

// Hinzufügen eines Admin-Menüs
function cci_add_admin_menu() {
    add_menu_page('CCSV Im- & Export', 'CSV Im- & Export', 'manage_options', 'csv_import_export', 'cci_csv_importer_exporter_page');
}
add_action('admin_menu', 'cci_add_admin_menu');

// Admin-Seiten-Layout
function cci_csv_importer_exporter_page() {
    ?>
    <div class="wrap">
        <h1>CSV Im- & Export</h1>

        <!-- Importbereich -->
        <h2>Import</h2>
        <form method="post" enctype="multipart/form-data">
            <?php wp_nonce_field('cci_csv_import', 'cci_csv_import_nonce'); ?>

            <table class="form-table">
                <tr valign="top">
                    <th scope="row"><label for="post_type">Post Type auswählen:</label></th>
                    <td>
                        <select name="post_type" id="post_type" required>
                            <?php
                            $post_types = get_post_types(['public' => true], 'objects');
                            foreach ($post_types as $post_type) {
                                echo '<option value="' . esc_attr($post_type->name) . '">' . esc_html($post_type->label) . '</option>';
                            }
                            ?>
                        </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="import_target">Importziel auswählen:</label></th>
                    <td>
                        <select name="import_target" id="import_target" required>
                            <option value="wp_custom_fields">WordPress Custom Fields</option>
                            <option value="cfs">Custom Field Suite (CFS)</option>
                        </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="csv_file">CSV-Datei hochladen:</label></th>
                    <td>
                        <input type="file" name="csv_file" id="csv_file" accept=".csv" required />
                    </td>
                </tr>
            </table>

            <?php submit_button('Importieren'); ?>
        </form>

        <!-- Exportbereich -->
        <h2>Export</h2>
        <form method="post" action="">
            <?php wp_nonce_field('cci_csv_export', 'cci_csv_export_nonce'); ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row"><label for="export_post_type">Post Type auswählen:</label></th>
                    <td>
                        <select name="export_post_type" id="export_post_type" required>
                            <?php
                            foreach ($post_types as $post_type) {
                                echo '<option value="' . esc_attr($post_type->name) . '">' . esc_html($post_type->label) . '</option>';
                            }
                            ?>
                        </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="meta_fields">Meta-Felder auswählen:</label></th>
                    <td id="meta_fields_container">
                        <!-- Meta-Felder werden dynamisch durch AJAX geladen -->
                    </td>
                </tr>
            </table>

            <?php submit_button('Exportieren'); ?>
        </form>
    </div>

    <script type="text/javascript">
        jQuery(document).ready(function($) {
            $('#export_post_type').change(function() {
                var postType = $(this).val();
                $.post(ajaxurl, {
                    action: 'cci_load_meta_fields',
                    post_type: postType
                }, function(response) {
                    $('#meta_fields_container').html(response);
                });
            });
        });
    </script>
    <?php
}

// AJAX-Handler für das Laden der Meta-Felder basierend auf dem ausgewählten Post-Type
function cci_load_meta_fields() {
    if (isset($_POST['post_type'])) {
        $post_type = sanitize_text_field($_POST['post_type']);
        $meta_keys = cci_get_all_meta_keys($post_type);
        
        foreach ($meta_keys as $meta_key) {
            if (substr($meta_key, 0, 1) !== '_') { // Nur Meta-Felder anzeigen, die nicht mit "_" beginnen
                echo '<label><input type="checkbox" name="meta_fields[]" value="' . esc_attr($meta_key) . '"> ' . esc_html($meta_key) . '</label><br>';
            }
        }
    }
    wp_die();
}
add_action('wp_ajax_cci_load_meta_fields', 'cci_load_meta_fields');

// Funktion zum Abrufen aller Meta-Felder eines Post-Types, die nicht mit "_" beginnen
function cci_get_all_meta_keys($post_type) {
    global $wpdb;
    $meta_keys = $wpdb->get_col($wpdb->prepare("
        SELECT DISTINCT meta_key 
        FROM {$wpdb->postmeta} 
        JOIN {$wpdb->posts} ON {$wpdb->posts}.ID = {$wpdb->postmeta}.post_id 
        WHERE {$wpdb->posts}.post_type = %s
        AND meta_key NOT LIKE '\_%'
    ", $post_type));
    return $meta_keys;
}

// CSV-Export-Funktion
// CSV-Export-Funktion
function cci_export_to_csv() {
    if (isset($_POST['cci_csv_export_nonce']) && wp_verify_nonce($_POST['cci_csv_export_nonce'], 'cci_csv_export')) {
        $post_type = sanitize_text_field($_POST['export_post_type']);
        $meta_fields = isset($_POST['meta_fields']) ? array_map('sanitize_text_field', $_POST['meta_fields']) : [];

        $filename = $post_type . '_export_' . date('Y-m-d') . '.csv';

        header('Content-Type: text/csv; charset=utf-8');
        header('Content-Disposition: attachment; filename=' . $filename);

        $output = fopen('php://output', 'w');

        // Kopfzeile in die CSV schreiben
        $headers = array_merge(['id', 'title', 'content', 'featured_image', 'categories', 'tags'], $meta_fields);
        fputcsv($output, $headers);

        // Beiträge des ausgewählten Post-Types holen
        $query = new WP_Query([
            'post_type' => $post_type,
            'posts_per_page' => -1,
        ]);

        while ($query->have_posts()) {
            $query->the_post();
            $row = [
                get_the_ID(),
                get_the_title(),
                get_the_content(),
                wp_get_attachment_url(get_post_thumbnail_id()), // Beitragsbild-URL
                implode(', ', wp_get_post_terms(get_the_ID(), 'category', ['fields' => 'names'])), // Kategorien
                implode(', ', wp_get_post_terms(get_the_ID(), 'post_tag', ['fields' => 'names'])), // Tags
            ];

            // Meta-Felderwerte hinzufügen
            foreach ($meta_fields as $meta_field) {
                $row[] = get_post_meta(get_the_ID(), $meta_field, true);
            }

            fputcsv($output, $row);
        }

        fclose($output);
        wp_reset_postdata();
        exit;
    }
}
add_action('admin_init', 'cci_export_to_csv');
